unit shdrdrop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls;

type
  TFileDroppedEvent = procedure(Sender: TObject; X,Y: Integer; filename: String) of object;

  TDDPDropped=TFileDroppedEvent;

  TDragDropPanel = class(TPanel)
  private
    { Private-Deklarationen }
    fActive: Boolean;
    fOnFileDropped: TDDPDropped;
    procedure WMDropFiles(var Message: TWMDropFiles); message WM_DROPFILES;
    procedure SetActive(val: Boolean);
  protected
    { Protected-Deklarationen }
    Procedure SetParent(Value: TWinControl); override;
  public
    { Public-Deklarationen }
    Constructor Create(AOwner: Tcomponent); override;
    Destructor Destroy; override;
  published
    { Published-Deklarationen }
    Property Active: Boolean read fActive write SetActive default False;
    property OnFileDropped: TDDPDropped read FOnFileDropped write FOnFileDropped;

    property Color;
    property Ctl3D;
    property Cursor;
    property Hint;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property ShowHint;
    property Visible;

    { Publish Inherited Protected Events }
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

Uses ShellAPI;

procedure Register;
begin
  RegisterComponents('Starwatch', [TDragDropPanel]);
end;

Constructor TDragDropPanel.Create(Aowner: TComponent);
Begin
     Inherited Create(AOwner);
     fActive:=False;
End;

Destructor TDragDropPanel.Destroy;
Begin
     If fActive and Not (csDesigning in ComponentState) And Assigned (Parent) Then DragAcceptFiles(Handle, False);
     Inherited Destroy;
End;

Procedure TDragDropPanel.WMDropFiles(var Message: TWMDropFiles);
var
  DropPt     : TPoint;
  NumDropped : Integer;
  DroppedName: Array [0..255] of Char;
  I          : Integer;
begin

     { This method was borrowed from the Borland Pascal 7.0 Shell Demo }

  // Get the mouse cursors coordinates

  DragQueryPoint(Message.Drop, DropPt);

  // How many files were dropped?

  NumDropped := DragQueryFile(Message.Drop, $FFFFFFFF, nil, 0);

  // Get the filenames.

  If Assigned(fOnFileDropped) Then // Anybody interested in the results?
  for I := 0 to NumDropped-1 do
  begin
    DragQueryFile(Message.Drop, I, DroppedName, SizeOf(DroppedName));
    fOnFileDropped(Self,DropPt.X,DropPt.Y,StrPas(DroppedName));
  end;

  // We're finished with this message
  DragFinish(Message.Drop);
end;

Procedure TDragDropPanel.SetActive(val: Boolean);
Begin
     fActive:=val;
     If Not Assigned(Parent) Then Exit; // Can't use DragAcceptFiles without Parent

     // I do not know whether it is safe to use DragAcceptFiles in design mode.
     // As it does not show effect while designing I simply do not call it

     If Not (csDesigning in ComponentState) Then DragAcceptFiles(Handle, fActive);
End;
Procedure TDragDropPanel.SetParent(Value: TWinControl);
Begin
     { This one was inspired by Brad Stowers' TGrabBar
       We have to override SetParent, because Parent is
       not valid at creation. As soon as we get Parent
       valid the component can start accepting files }

     // Leave the old parent clean
     If Assigned(Parent) And fActive Then DragAcceptFiles(Handle,False);

     // Let the TPanel.SetParent do its work
     inherited SetParent(Value);

     // Paranoia: Did we really get a valid parent handle?
     If Assigned(Parent) Then DragAcceptFiles(Handle,fActive);
End;

end.
